package com.idea.buildManage.controller;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.dto.MnpBuildingDto;
import com.idea.buildManage.excel.MnpBuildingExcel;
import com.idea.buildManage.excel.MnpBuildingListener;
import com.idea.buildManage.excel.ParkInfoExcel;
import com.idea.buildManage.service.ParkFloorDiscService;
import com.idea.buildManage.service.ParkInfoService;
import com.idea.buildManage.service.ParkRoomService;
import com.idea.buildManage.model.MnpBuilding;
import com.idea.buildManage.model.MnpBuildingExample;
import com.idea.buildManage.service.MnpBuildingService;
import com.idea.util.ExcelUtils;
import com.rockstar.common.base.BaseController;
import com.rockstar.common.domain.AjaxResult;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.TableSplitResult;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.system.service.SysDictService;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by pengyq on 2020.
 */
@Controller
@RequestMapping(value = "MnpBuildingController")
@Api(tags = "载体管理")
public class MnpBuildingController extends BaseController {

    //主表
    @Autowired
    private MnpBuildingService modelService;
    @Autowired
    private ParkFloorDiscService floorDiscService;
    @Autowired
    private ParkInfoService parkInfoService;
    @Autowired
    private SysDictService sysDictService;


    @PostMapping(value = "list", produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object list(Tablepar tablepar, MnpBuildingDto model, DateTrans dt) {
        PageInfo<MnpBuildingDto> page = modelService.list(tablepar, model, dt);
        TableSplitResult<MnpBuildingDto> result = new TableSplitResult<MnpBuildingDto>(page.getPageNum(), page.getTotal(), page.getList());
        return result;
    }


    @PostMapping(value = "listAll", produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object listAll(MnpBuilding model, DateTrans dt) {
        return modelService.listAll(model, dt);
    }

    @PostMapping(value = "add", produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult add(MnpBuilding record) {
        try {
            // 同一楼盘下 载体名称不能重复
            MnpBuildingExample mnpBuildingExample = new MnpBuildingExample();
            mnpBuildingExample.createCriteria().andBuildNumEqualTo(record.getBuildNum()).andDiscIdEqualTo(record.getDiscId());
            long count = modelService.countByExample(mnpBuildingExample);
            if (count > 0) {
                throw new RuntimeException(record.getBuildNum() + "已存在,请调整后重新添加!!!");
            }
            record.setCreatedId(ShiroUtils.getUser().getId());
            record.setCreatedAt(new Date());
            String orderNum = modelService.getOrderNum(record);
            record.setOrderNum(orderNum);
            int result = modelService.insertSelective(record);
            return result(result);
        } catch (Exception e) {
            return error(e.getMessage());
        }
    }

    @PostMapping(value = "remove/{id}", produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult remove(@PathVariable("id") String id) {
        int result = modelService.deleteByPrimaryKey(id);
        return result(result);
    }

    @PostMapping(value = "edit", produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult editSave(MnpBuilding model) {
        int result = modelService.updateByPrimaryKeySelective(model);
        return result(result);
    }

    @PostMapping(value = "getById", produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public MnpBuilding getById(String id) {
        return modelService.selectByPrimaryKey(id);
    }


    @PostMapping(value = "excelListAll", produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object excelListAll(MnpBuildingDto model) {

        return modelService.excelListAll(model);
    }

    /**
     * 楼栋导入
     * @param file
     * @return
     */
    @PostMapping(value = "handleImport", produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult handleImport(@RequestParam("file") MultipartFile file) {
        JSONObject jsonObject = new JSONObject();
        MnpBuildingListener listener = new MnpBuildingListener(modelService,floorDiscService,parkInfoService,sysDictService,
                jsonObject);
        try {
            ExcelUtils excelUtils = new ExcelUtils();
            File excel = excelUtils.multipartFileToFile(file);
            if (null == excel) {
                return AjaxResult.error("导入失败");
            }
            EasyExcel.read(excel, MnpBuildingExcel.class, listener).headRowNumber(1).sheet().doRead();
            excel.delete();
        } catch (Exception e) {
            e.printStackTrace();
            return AjaxResult.error(e.getMessage());
        }
        return AjaxResult.success(jsonObject);
    }

    /**
     * 楼栋删除
     * @param id
     * @return
     */
    @PostMapping(value = "delete", produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult delete(String id) {
        int result = modelService.delete(id);
        return result(result);
    }

    /**
     * 导出
     * @param list
     * @param response
     * @throws IOException
     */
    @PostMapping(value = "errorListExport")
    @ResponseBody
    public void errorListExport(@RequestBody List<MnpBuildingExcel> list, HttpServletResponse response) throws IOException {
        modelService.errorListExport(list,response);
    }





}
