package com.idea.buildManage.controller;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.excel.ParkFloorDiscExcel;
import com.idea.buildManage.excel.ParkFloorDiscListener;
import com.idea.buildManage.excel.ParkInfoExcel;
import com.idea.buildManage.excel.ParkInfoListener;
import com.idea.buildManage.model.ParkInfo;
import com.idea.buildManage.service.ParkInfoService;
import com.idea.util.ExcelUtils;
import com.rockstar.common.base.BaseController;
import com.rockstar.common.domain.AjaxResult;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.TableSplitResult;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.system.service.SysDictService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.List;

/**
 * Created by pengyq on 2020.
 */
@Controller
@RequestMapping(value = "ParkInfoController")
@Api(value="园区管理")
public class ParkInfoController extends BaseController {

    //主表
    @Autowired
    private ParkInfoService modelService;
    @Autowired
    private SysDictService sysDictService;


    @PostMapping(value = "list",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object list(Tablepar tablepar, ParkInfo model, DateTrans dt){
        PageInfo<ParkInfo> page= modelService.list(tablepar,model,dt);
        TableSplitResult<ParkInfo> result = new TableSplitResult<ParkInfo>(page.getPageNum(), page.getTotal(), page.getList());
        return  result;
    }

    @PostMapping(value = "listAll",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object listAll(ParkInfo model, DateTrans dt){
        return  modelService.listAll(model,dt);
    }

    @PostMapping(value = "add",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult add(ParkInfo record){
        record.setCreatedAt(new Date());
        int result = modelService.insert(record);
        return result(result);
    }

    @PostMapping(value = "remove/{id}",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult remove(@PathVariable("id") String id){
        int result = modelService.deleteByPrimaryKey(id);
        return result(result);
    }

    @PostMapping(value = "edit",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult editSave(ParkInfo model){
        model.setUpdatedAt(new Date());
        int result = modelService.updateByPrimaryKeySelective(model);
        return result(result);
    }

    @PostMapping(value = "getById",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public ParkInfo getById(String id) {
        return modelService.selectByPrimaryKey(id);
    }

    @PostMapping(value = "addModel",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult addModel(ParkInfo record){
        record.setCreatedAt(new Date());
        int result = modelService.insertModel(record);
        return result(result);
    }

//    @PostMapping(value = "delete/{id}",produces = {"application/json;charset=UTF-8"})
//    @ResponseBody
//    public AjaxResult delete(@PathVariable("id") String id){
//        int result = modelService.deleteByKey(id);
//        return result(result);
//    }

    //获取属性结构1
    @PostMapping(value = "getTreeData1",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult getTreeData1(){
        return success(modelService.getTreeData_1());
    }

    //获取属性结构2
    @PostMapping(value = "getTreeData2",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult getTreeData2(){
        return success(modelService.getTreeData_2());
    }

    //获取属性结构3
    @PostMapping(value = "getTreeData3",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult getTreeData3(){
        return success(modelService.getTreeData_3());
    }

    //获取楼盘信息3
    @PostMapping(value = "getlouDetial",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult getlouDetial(String ids){
        return success(modelService.getLouDetial(ids));
    }

    @PostMapping(value = "getByIds",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult getByIds(String ids) {
        return success(modelService.selectByIds(ids));
    }

    /**
     * 楼盘导入
     * @param file
     * @return
     */
    @PostMapping(value = "handleImport", produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult handleImport(@RequestParam("file") MultipartFile file) {
        JSONObject jsonObject = new JSONObject();
        ParkInfoListener listener = new ParkInfoListener(modelService,sysDictService,jsonObject);
        try {
            ExcelUtils excelUtils = new ExcelUtils();
            File excel = excelUtils.multipartFileToFile(file);
            if (null == excel) {
                return AjaxResult.error("导入失败");
            }
            EasyExcel.read(excel, ParkInfoExcel.class, listener).headRowNumber(1).sheet().doRead();
            excel.delete();
        } catch (Exception e) {
            e.printStackTrace();
            return AjaxResult.error(e.getMessage());
        }
        return AjaxResult.success(jsonObject);
    }

    @ApiOperation(value = "园区删除")
    @PostMapping(value = "delete",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult delete(String id) {
        int result = modelService.delete(id);
        return result(result);
    }

    /**
     * 导出
     * @param list
     * @param response
     * @throws IOException
     */
    @PostMapping(value = "errorListExport")
    @ResponseBody
    public void errorListExport(@RequestBody List<ParkInfoExcel> list, HttpServletResponse response) throws IOException {
        modelService.errorListExport(list,response);
    }


}
