package com.idea.buildManage.excel;

import cn.hutool.core.util.IdUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson.JSONObject;
import com.idea.buildManage.model.ParkFloorDisc;
import com.idea.buildManage.service.ParkFloorDiscService;
import com.idea.buildManage.service.ParkInfoService;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.StringUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

public class ParkFloorDiscListener extends AnalysisEventListener<ParkFloorDiscExcel> {

    private List<ParkFloorDisc> list = new ArrayList<>();

    private List<ParkFloorDiscExcel> excelList = new ArrayList<>();

    private List<ParkFloorDiscExcel> badList = new ArrayList<>();

    Integer count;

    private JSONObject jsonObject;

    private ParkFloorDiscService floorDiscService;
    private ParkInfoService parkInfoService;
    private SysDictService sysDictService;


    public ParkFloorDiscListener(ParkFloorDiscService floorDiscService, ParkInfoService parkInfoService,
                                 SysDictService sysDictService, JSONObject jsonObject) {
        this.floorDiscService = floorDiscService;
        this.parkInfoService = parkInfoService;
        this.sysDictService = sysDictService;
        this.jsonObject = jsonObject;
    }


    @Override
    public void invoke(ParkFloorDiscExcel parkFloorDiscExcel, AnalysisContext analysisContext) {
        excelList.add(parkFloorDiscExcel);
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {


        // 支付方式字典
        List<Map<String, String>> paymentMethods = sysDictService.selectDictList("PAYMENT_METHODS");

        if (CollectionUtils.isNotEmpty(excelList)) {
            ParkFloorDisc data;
            ParkFloorDiscExcel excel = null;
            Map<String, String> discNameIdMap = floorDiscService.getNameIdMap();
            Map<String, String> parkNameIdMap = parkInfoService.getNameIdMap();


            for (int i = 0; i < excelList.size(); i++) {
                try {

                    boolean groupNameFlag = true;
                    boolean discNameFlag = true;
                    boolean unitPriceFlag = true;
                    boolean propertyUnitPriceFlag = true;
                    boolean maintenanceFundsFlag = true;
                    boolean subscriptionFundsFlag = true;
                    boolean proportionFlag = true;
                    boolean paymentMethodsFlag = true;
                    StringBuilder builder = new StringBuilder();

                    data = new ParkFloorDisc();
                    excel = excelList.get(i);
                    if (StringUtils.isEmpty(excel.getGroupName())) {

                        groupNameFlag = false;
                        builder.append("楼盘/小区名称为空").append(",");
                        excel.setGroupName("{" + excel.getGroupName() + "}");
                    }
                    if (StringUtils.isEmpty(excel.getDiscName())) {
                        discNameFlag = false;
                        builder.append("楼分期名称为空").append(",");
                        excel.setDiscName("{" + excel.getDiscName() + "}");
                    }
                    data.setName(excel.getDiscName());
                    // 判断小区是否存在
                    if (StringUtils.isEmpty(parkNameIdMap.get(excel.getGroupName()))) {
                        groupNameFlag = false;
                        builder.append("楼盘/小区不存在").append(",");
                        excel.setGroupName("{" + excel.getGroupName() + "}");
                    }
                    data.setGroupId(parkNameIdMap.get(excel.getGroupName()));

                    // 房屋买卖单价
                    if (StringUtils.isNotEmpty(excel.getUnitPrice())) {
                        try {
                            data.setUnitPrice(new BigDecimal(excel.getUnitPrice()));
                        } catch (NumberFormatException e) {
                            unitPriceFlag = false;
                            builder.append("房屋买卖单价（元/㎡）格式有误").append(",");
                            excel.setUnitPrice("{" + excel.getUnitPrice() + "}");
                        }
                    }

                    // 物业费单价
                    if (StringUtils.isNotEmpty(excel.getPropertyUnitPrice())) {
                        try {
                            data.setPropertyUnitPrice(new BigDecimal(excel.getPropertyUnitPrice()));
                        } catch (NumberFormatException e) {
                            propertyUnitPriceFlag = false;
                            builder.append("物业费单价（元/㎡）格式有误").append(",");
                            excel.setPropertyUnitPrice("{" + excel.getPropertyUnitPrice() + "}");
                        }
                    }

                    // 住宅专项维修资金
                    if (StringUtils.isNotEmpty(excel.getMaintenanceFunds())) {
                        try {
                            data.setMaintenanceFunds(new BigDecimal(excel.getMaintenanceFunds()));
                        } catch (NumberFormatException e) {
                            maintenanceFundsFlag = false;
                            builder.append("住宅专项维修资金（元/㎡）格式有误").append(",");
                            excel.setMaintenanceFunds("{" + excel.getMaintenanceFunds() + "}");
                        }
                    }

                    // 认购金
                    if (StringUtils.isNotEmpty(excel.getSubscriptionFunds())) {
                        try {
                            data.setSubscriptionFunds(new BigDecimal(excel.getSubscriptionFunds()));
                        } catch (NumberFormatException e) {
                            subscriptionFundsFlag = false;
                            builder.append("认购金（元）格式有误").append(",");
                            excel.setSubscriptionFunds("{" + excel.getSubscriptionFunds() + "}");
                        }
                    }

                    // 共持比例
                    if (StringUtils.isNotEmpty(excel.getProportion())) {
                        try {
                            data.setProportion(Double.valueOf(excel.getProportion()));
                        } catch (NumberFormatException e) {
                            proportionFlag = false;
                            builder.append("共持比例格式有误").append(",");
                            excel.setProportion("{" + excel.getProportion() + "}");
                        }
                    }

//                    // 支付方式
//                    if (StringUtils.isNotEmpty(excel.getPaymentMethods())) {
//                        for (Map<String, String> stringStringMap : paymentMethods) {
//                            if (stringStringMap.get("label").equals(excel.getPaymentMethods())) {
//                                data.setPaymentMethods(Integer.valueOf(stringStringMap.get("value")));
//                            }
//                        }
//                        // 没匹配上的情况
//                        if (data.getPaymentMethods() == null) {
//                            paymentMethodsFlag = false;
//                            builder.append("支付方式格式有误").append(",");
////                            throw new RuntimeException("支付方式格式有误");
//                        }
//                    }

                    if (groupNameFlag && discNameFlag && unitPriceFlag && propertyUnitPriceFlag && maintenanceFundsFlag
                            && subscriptionFundsFlag && proportionFlag && paymentMethodsFlag
                    ) {
                        data.setId(IdUtil.simpleUUID());
                        data.setCreatedId(ShiroUtils.getUserId());
                        data.setCreatedAt(new Date());
                        list.add(data);
                    }else {
                        throw new RuntimeException(builder.toString());
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    String message = e.getMessage();
                    excel.setReason(message);
                    badList.add(excel);
                }
            }

            count = list.size();
            jsonObject.put("successCount", count);
            jsonObject.put("errorList", badList);
            jsonObject.put("errorCount", badList.size());
            //保存
            saveData();

            excelList.clear();
            list.clear();

        } else {
            throw new RuntimeException("excel无数据");
        }


    }

    private void saveData() {
        if (CollectionUtils.isNotEmpty(list)) {
            floorDiscService.insertBatch(list);
        }
    }
}
