package com.idea.buildManage.service;

import com.alibaba.excel.EasyExcel;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.excel.ParkFloorDiscExcel;
import com.idea.buildManage.excel.ParkInfoExcel;
import com.idea.buildManage.excel.WriteHandlerStrategy;
import com.idea.buildManage.mapper.ParkFloorDiscExtendMapper;
import com.idea.buildManage.mapper.ParkFloorDiscMapper;
import com.idea.buildManage.mapper.ParkInfoExtendMapper;
import com.idea.buildManage.mapper.ParkInfoMapper;
import com.idea.buildManage.model.*;
import com.rockstar.common.base.BaseService;
import com.rockstar.common.support.Convert;
import com.rockstar.frame.model.FrameUser;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.system.mapper.SysGroupMapper;
import com.rockstar.system.model.SysGroup;
import com.rockstar.system.model.SysGroupExample;
import com.rockstar.system.service.SysGroupService;
import com.rockstar.util.SnowflakeIdWorker;
import com.rockstar.util.StringUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.beans.Transient;
import java.io.IOException;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class ParkInfoService implements BaseService<ParkInfo, ParkInfoExample> {

    //文件mapper
    @Autowired
    private ParkInfoMapper modelMapper;
    @Autowired
    private SysGroupService sysGroupService;
    @Autowired
    private SysGroupMapper sysGroupMapper;
    @Autowired
    private ParkFloorDiscMapper parkFloorDiscMapper;
    @Autowired
    private ParkRoomService parkRoomService;
    @Autowired
    private MnpBuildingService mnpBuildingService;
    @Autowired
    private ParkFloorDiscExtendMapper extendMapper;
    @Autowired
    private ParkInfoExtendMapper infoExtendMapper;

    /**
     * 分页查询
     *
     * @return
     */
    public PageInfo<ParkInfo> list(Tablepar tablepar, ParkInfo model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<ParkInfo> list = modelMapper.selectByExample(getCondition(model, dt));
        PageInfo<ParkInfo> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 查询指定
     *
     * @return
     */
    public List<ParkInfo> listAll(ParkInfo model, DateTrans dt) {
        return modelMapper.selectByExample(getCondition(model, dt));
    }

    private ParkInfoExample getCondition(ParkInfo model, DateTrans dt) {
        ParkInfoExample ex = new ParkInfoExample();
        ex.setOrderByClause("created_at desc");
        ParkInfoExample.Criteria criteria = ex.createCriteria();
        return ex;
    }

    @Override
    public int deleteByPrimaryKey(String id) {

        //查询园区下是否存在楼盘
        ParkFloorDiscExample parkFloorDiscExample = new ParkFloorDiscExample();
        parkFloorDiscExample.createCriteria().andGroupIdEqualTo(id);
        long count = parkFloorDiscMapper.countByExample(parkFloorDiscExample);
        if (count > 0) {
            throw new RuntimeException("该园区下存在楼盘,无法删除!");
        }
        return modelMapper.deleteByPrimaryKey(id);
    }

    @Override
    public ParkInfo selectByPrimaryKey(String id) {
        return modelMapper.selectByPrimaryKey(id);
    }

    public Map<String, String> selectByIds(String ids) {
        Map<String, String> map = new HashMap<>();
        for (String id : ids.split(",")) {
            if (StringUtils.isNotEmpty(id)) {
                ParkInfo model = modelMapper.selectByPrimaryKey(id);
                if (null != model) {
                    map.put(id, model.getGroupName());
                } else {
                    map.put(id, "");
                }
            }
        }
        return map;
    }

    @Override
    public int updateByPrimaryKeySelective(ParkInfo record) {
        record.setUpdatedAt(new Date());
        return modelMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int updateByExampleSelective(ParkInfo record, ParkInfoExample example) {
        record.setUpdatedAt(new Date());
        return modelMapper.updateByExampleSelective(record, example);
    }

    @Override
    public int updateByExample(ParkInfo record, ParkInfoExample example) {
        record.setUpdatedAt(new Date());
        return modelMapper.updateByExample(record, example);
    }

    @Override
    public List<ParkInfo> selectByExample(ParkInfoExample example) {
        return modelMapper.selectByExample(example);
    }

    @Override
    public long countByExample(ParkInfoExample example) {
        return modelMapper.countByExample(example);
    }

    public long countByExample(ParkInfo model, DateTrans dt) {
        return modelMapper.countByExample(getCondition(model, dt));
    }

    @Override
    public int deleteByExample(ParkInfoExample example) {
        return modelMapper.deleteByExample(example);
    }

    public int insert(ParkInfo record) {
        record.setId(SnowflakeIdWorker.getUUID());
        return insertWithoutId(record);
    }

    public int insertWithoutId(ParkInfo record) {
        record.setCreatedAt(new Date());
        return modelMapper.insert(record);
    }

    @Override
    public int insertSelective(ParkInfo record) {
        record.setId(SnowflakeIdWorker.getUUID());
        record.setCreatedAt(new Date());
        return modelMapper.insertSelective(record);
    }

    @Transient
    public int insertModel(ParkInfo record) {
        if (StringUtils.isNotEmpty(record.getId())) {
            record.setUpdatedAt(new Date());
            modelMapper.updateByPrimaryKeyWithBLOBs(record);
            SysGroup group = sysGroupMapper.selectByPrimaryKey(record.getGroupId());
            group.setName(record.getGroupName());
            group.setUpdatedAt(new Date());
            sysGroupMapper.updateByPrimaryKey(group);
        } else {
            FrameUser user = ShiroUtils.getUser();
            String id = SnowflakeIdWorker.getUUID();
            SysGroup group = new SysGroup();
            group.setId(id);
            group.setCreatedBy(user.getId());
            group.setName(record.getGroupName());
            group.setCreatedAt(new Date());
            sysGroupMapper.insert(group);
            record.setId(id);
            record.setGroupId(id);
            record.setCreatedId(user.getId());
            String orderNum = getOrderNum(record);
            record.setOrderNum(orderNum);
            record.setUpdatedAt(new Date());
            insertWithoutId(record);
        }
        return 1;
    }

    @Transient
    public int deleteByKey(String ids) {
        List<String> lista = Convert.toListStrArray(ids);
        ParkInfoExample example = new ParkInfoExample();
        example.createCriteria().andIdIn(lista);
        modelMapper.deleteByExample(example);

        SysGroupExample sge = new SysGroupExample();
        sge.createCriteria().andIdIn(lista);
        sysGroupService.deleteByExample(sge);
        return 2;
    }

    public List<Map<String, Object>> getTreeData_1() {
        List<ParkInfo> infos = listAll(null, null);
        List<Map<String, Object>> treeData = new ArrayList<>();
        for (ParkInfo info : infos) {
            Map<String, Object> map = new HashMap<>();
            map.put("label", info.getGroupName());
            map.put("id", info.getId());
            map.put("value", info.getId());
            //楼盘
            List<ParkFloorDisc> parkFloorDiscList = extendMapper.selectByGroupId(info.getId());
            if (null != parkFloorDiscList && parkFloorDiscList.size() > 0) {
                List<Map<String, Object>> childrens = new ArrayList<>();
                for (ParkFloorDisc parkFloorDisc : parkFloorDiscList) {
                    Map<String, Object> children = new HashMap<>();
                    children.put("label", parkFloorDisc.getName());
                    children.put("id", parkFloorDisc.getId());
                    children.put("value", parkFloorDisc.getId());
                    //楼栋
                    MnpBuildingExample buildingExample = new MnpBuildingExample();
                    buildingExample.setOrderByClause("created_at desc");
                    buildingExample.createCriteria().andDiscIdEqualTo(parkFloorDisc.getId());
                    List<MnpBuilding> parkBuilds = mnpBuildingService.selectByExample(buildingExample);
                    if (null != parkBuilds && parkBuilds.size() > 0) {
                        List<Map<String, Object>> parkBuildChildrens = new ArrayList<>();
                        for (MnpBuilding parkBuild : parkBuilds) {
                            Map<String, Object> parkBuildChildren = new HashMap<>();
                            parkBuildChildren.put("label", parkBuild.getBuildNum());
                            parkBuildChildren.put("id", parkBuild.getId());
                            parkBuildChildren.put("value", parkBuild.getId());
                            //房间
                            ParkRoomExample parkRoomExample = new ParkRoomExample();
                            parkRoomExample.setOrderByClause("created_at desc");
                            parkRoomExample.createCriteria().andBuildIdEqualTo(parkBuild.getId());
                            List<ParkRoom> parkRooms = parkRoomService.selectByExample(parkRoomExample);
                            if (null != parkRooms && parkRooms.size() > 0) {
                                List<Map<String, Object>> parkRoomChildrens = new ArrayList<>();
                                for (ParkRoom parkRoom : parkRooms) {
                                    Map<String, Object> parkRoomChildren = new HashMap<>();
                                    parkRoomChildren.put("label", parkRoom.getRoomNo());
                                    parkRoomChildren.put("id", parkRoom.getId());
                                    parkRoomChildren.put("value", parkRoom.getId());
                                    parkRoomChildrens.add(parkRoomChildren);
                                }
                                parkBuildChildren.put("children", parkRoomChildrens);
                            }

                            parkBuildChildrens.add(parkBuildChildren);
                        }
                        children.put("children", parkBuildChildrens);
                    }
                    childrens.add(children);
                }
                map.put("children", childrens);
            }
            treeData.add(map);
        }
        return treeData;
    }

    public List<Map<String, Object>> getTreeData_2() {
        List<ParkInfo> infos = listAll(null, null);
        List<Map<String, Object>> treeData = new ArrayList<>();
        for (ParkInfo info : infos) {
            Map<String, Object> map = new HashMap<>();
            map.put("label", info.getGroupName());
            map.put("id", info.getId());
            map.put("value", info.getId());
            //楼盘
            List<ParkFloorDisc> parkFloorDiscList = extendMapper.selectByGroupId(info.getId());
            if (null != parkFloorDiscList && parkFloorDiscList.size() > 0) {
                List<Map<String, Object>> childrens = new ArrayList<>();
                for (ParkFloorDisc parkFloorDisc : parkFloorDiscList) {
                    Map<String, Object> children = new HashMap<>();
                    children.put("label", parkFloorDisc.getName());
                    children.put("id", parkFloorDisc.getId());
                    children.put("value", parkFloorDisc.getId());
                    //楼栋
                    MnpBuildingExample buildingExample = new MnpBuildingExample();
                    buildingExample.setOrderByClause("created_at desc");
                    buildingExample.createCriteria().andDiscIdEqualTo(parkFloorDisc.getId());
                    List<MnpBuilding> parkBuilds = mnpBuildingService.selectByExample(buildingExample);
                    if (null != parkBuilds && parkBuilds.size() > 0) {
                        List<Map<String, Object>> parkBuildChildrens = new ArrayList<>();
                        for (MnpBuilding parkBuild : parkBuilds) {
                            Map<String, Object> parkBuildChildren = new HashMap<>();
                            parkBuildChildren.put("label", parkBuild.getBuildNum());
                            parkBuildChildren.put("id", parkBuild.getId());
                            parkBuildChildren.put("value", parkBuild.getId());
                            parkBuildChildrens.add(parkBuildChildren);
                        }
                        children.put("children", parkBuildChildrens);
                    }
                    childrens.add(children);
                }
                map.put("children", childrens);
            }
            treeData.add(map);
        }
        return treeData;
    }

    public List<Map<String, Object>> getTreeData_3() {
        List<ParkInfo> infos = listAll(null, null);
        List<Map<String, Object>> treeData = new ArrayList<>();
        for (ParkInfo info : infos) {
            Map<String, Object> map = new HashMap<>();
            map.put("label", info.getGroupName());
            map.put("id", info.getId());
            map.put("value", info.getId());
            //楼盘
            List<ParkFloorDisc> parkFloorDiscList = extendMapper.selectByGroupId(info.getId());
            if (null != parkFloorDiscList && parkFloorDiscList.size() > 0) {
                List<Map<String, Object>> childrens = new ArrayList<>();
                for (ParkFloorDisc parkFloorDisc : parkFloorDiscList) {
                    Map<String, Object> children = new HashMap<>();
                    children.put("label", parkFloorDisc.getName());
                    children.put("id", parkFloorDisc.getId());
                    children.put("value", parkFloorDisc.getId());
                    childrens.add(children);
                }
                map.put("children", childrens);
            }
            treeData.add(map);
        }
        return treeData;
    }

    public Map<String, String> getLouDetial(String ids) {
        Map<String, String> map = new HashMap<>();
        for (String id : ids.split(",")) {
            if (StringUtils.isNotEmpty(id)) {
                String lous = "";
                List<ParkFloorDisc> list = extendMapper.selectByGroupId(id);
                for (ParkFloorDisc parkFloorDisc : list) {
                    lous += parkFloorDisc.getName() + ",";
                }
                if (StringUtils.isNotEmpty(lous)) {
//                return lous.substring(0, lous.length()-1);
                    map.put(id, lous.substring(0, lous.length() - 1));
                } else {
                    map.put(id, lous);
                }
            }
        }
        return map;
    }

    public Map<String,String> getNameIdMap() {
        Map<String, String> map = new HashMap<>();
        List<ParkInfo> list = selectByExample(new ParkInfoExample());
        if (CollectionUtils.isNotEmpty(list)) {
            map = list.stream().collect(Collectors.toMap(ParkInfo::getGroupName, ParkInfo::getId, (v1, v2) ->v1));
        }
        return map;
    }

    /**
     * 批量导入
     * @param list
     */
    public void insertBatch(List<ParkInfo> list) {
        infoExtendMapper.insertBatch(list);
    }

    /**
     * 园区删除
     * @param id
     * @return
     */
    public int delete(String id){

        ParkFloorDiscExample floorDiscExample = new ParkFloorDiscExample();
        floorDiscExample.createCriteria().andGroupIdEqualTo(id);
        List<ParkFloorDisc> parkFloorDiscs = parkFloorDiscMapper.selectByExample(floorDiscExample);
        if(CollectionUtils.isNotEmpty(parkFloorDiscs)){
            throw new RuntimeException("该数据有子分期存在，无法删除");
        }
        return deleteByPrimaryKey(id);
    }

    /**
     * 获取排序号
     * @param model
     * @return
     */
    private String getOrderNum(ParkInfo model){

        // 排序号
        Long count = countByExample(new ParkInfoExample());
        String number = "";
        if(count.doubleValue() == 0){
            number = "01";
        }else {
            count++;
            number = String.format("%02d", count);
        }
        return number;
    }



    public void errorListExport(List<ParkInfoExcel> list, HttpServletResponse response) throws IOException {


        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String date = sdf.format(new Date());
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        String fileName = URLEncoder.encode("失败记录" + date, "UTF-8");
        response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
        EasyExcel.write(response.getOutputStream(), ParkInfoExcel.class)
                .inMemory(true)
                .registerWriteHandler(new WriteHandlerStrategy())//自定义样式
                .sheet().doWrite(list);

    }



}
