package com.idea.customerManagement.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.mapper.ParkInfoMapper;
import com.idea.buildManage.mapper.ParkRoomMapper;
import com.idea.buildManage.model.ParkInfo;
import com.idea.buildManage.model.ParkRoom;
import com.idea.buildManage.response.ParkRoomResponse;
import com.idea.customerManagement.dto.CustomerManagementDto;
import com.idea.customerManagement.dto.RoomSelectionInfoDto;
import com.idea.customerManagement.mapper.CustomerManagementExtendMapper;
import com.idea.customerManagement.mapper.CustomerManagementMapper;
import com.idea.customerManagement.mapper.RoomSelectionInfoMapper;
import com.idea.customerManagement.model.*;
import com.idea.util.DateUtils;
import com.idea.util.ReplaceWord;
import com.rockstar.common.base.BaseService;
import com.rockstar.frame.model.FrameUser;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.frame.service.FrameUserService;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.StringUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.*;

@Service
public class CustomerManagementService implements BaseService<CustomerManagement, CustomerManagementExample> {


    @Autowired
    private CustomerManagementMapper modelMapper;
    @Autowired
    private CustomerManagementExtendMapper extendMapper;
    @Autowired
    private BuyerService buyerService;
    @Autowired
    private SysDictService sysDictService;
    @Autowired
    private FrameUserService frameUserService;
    @Autowired
    private RoomSelectionInfoMapper roomSelectionInfoMapper;
    @Autowired
    private ParkRoomMapper parkRoomMapper;
    @Autowired
    private ParkInfoMapper parkInfoMapper;

    @Override
    public int deleteByPrimaryKey(String s) {
        return modelMapper.deleteByPrimaryKey(s);
    }

    @Override
    public int insertSelective(CustomerManagement customerManagement) {
        customerManagement.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(customerManagement);
    }

    public int insertWithOutId(CustomerManagement customerManagement) {
        return modelMapper.insert(customerManagement);
    }

    @Override
    public CustomerManagement selectByPrimaryKey(String s) {
        return modelMapper.selectByPrimaryKey(s);
    }

    @Override
    public int updateByPrimaryKeySelective(CustomerManagement customerManagement) {
        return modelMapper.updateByPrimaryKeySelective(customerManagement);
    }

    @Override
    public int updateByExampleSelective(CustomerManagement customerManagement, CustomerManagementExample customerManagementExample) {
        return modelMapper.updateByExampleSelective(customerManagement,customerManagementExample);
    }

    @Override
    public int updateByExample(CustomerManagement customerManagement, CustomerManagementExample customerManagementExample) {
        return modelMapper.updateByExample(customerManagement,customerManagementExample);
    }

    @Override
    public List<CustomerManagement> selectByExample(CustomerManagementExample customerManagementExample) {
        return modelMapper.selectByExample(customerManagementExample);
    }

    @Override
    public long countByExample(CustomerManagementExample customerManagementExample) {
        return modelMapper.countByExample(customerManagementExample);
    }

    @Override
    public int deleteByExample(CustomerManagementExample customerManagementExample) {
        return modelMapper.deleteByExample(customerManagementExample);
    }

    /**
     * 分页查询
     *
     * @return
     */
    public PageInfo<CustomerManagementDto> listByModel(Tablepar tablepar, CustomerManagementDto model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        if(StringUtils.isNotEmpty(model.getGroupIds())){
            model.setGroupIdList(Arrays.asList(model.getGroupIds().split(",")));
        }
        if(StringUtils.isNotEmpty(model.getDiscIds())){
            model.setDiscIdList(Arrays.asList(model.getDiscIds().split(",")));
        }
        List<CustomerManagementDto> list = extendMapper.listByModel(model);
        PageInfo<CustomerManagementDto> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 新增轮号客户
     * @param model
     * @return
     */
    public int add(CustomerManagementDto model){

        CustomerManagementExample example = new CustomerManagementExample();
        example.createCriteria().andBatchNumberEqualTo(model.getBatchNumber())
                .andRoomSelectionNumberEqualTo(model.getRoomSelectionNumber());
        long count = countByExample(example);
        if(count>0){
            throw new RuntimeException("批次号选房号重复!!!");
        }

        String buyerJson = model.getBuyerJson();
        String id = IdUtil.simpleUUID();
        String userId = ShiroUtils.getUserId();
        Date date = new Date();
        List<Buyer> buyers = JSONObject.parseArray(buyerJson, Buyer.class);
        int i = 1;
        StringBuilder builder = new StringBuilder();
        for(Buyer buyer : buyers){
            buyer.setOrderNum(i);
            i++;
            buyer.setCustomerManagementId(id);
            buyer.setCreatedAt(date);
            buyer.setCreatedId(userId);
            buyerService.insertSelective(buyer);
            if(StringUtils.isEmpty(buyer.getRelationship())){
                builder.append(buyer.getName()).append(",");
            }else {
                String dictLabel = sysDictService.getDictLabel("RELATIONSHIP", buyer.getRelationship());
                builder.append(buyer.getName())
                        .append("(")
                        .append(dictLabel)
                        .append(")")
                        .append(",");
            }
        }
        if(builder.length() > 0){
            builder.deleteCharAt(builder.length()-1);
        }
        model.setBuyerName(builder.toString());
        model.setId(id);
        // 待选房
        model.setStatus(1);
        model.setCreatedAt(date);
        model.setCreatedId(userId);
        return insertWithOutId(model);

    }


    /**
     * 编辑轮号客户
     * @param model
     * @return
     */
    public int edit(CustomerManagementDto model){

        String buyerJson = model.getBuyerJson();
        String id = model.getId();
        String userId = ShiroUtils.getUserId();
        Date date = new Date();

        // 先删除
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.createCriteria().andCustomerManagementIdEqualTo(id);
        List<Buyer> oldBuyers = buyerService.selectByExample(buyerExample);
        for(Buyer buyer : oldBuyers){
            buyerService.deleteByPrimaryKey(buyer.getId());
        }
        // 再新增
        List<Buyer> buyers = JSONObject.parseArray(buyerJson, Buyer.class);
        int i = 1;
        StringBuilder builder = new StringBuilder();
        for(Buyer buyer : buyers){
            buyer.setOrderNum(i);
            i++;
            buyer.setCustomerManagementId(id);
            buyer.setCreatedAt(date);
            buyer.setCreatedId(userId);
            buyerService.insertSelective(buyer);
            if(StringUtils.isEmpty(buyer.getRelationship())){
                builder.append(buyer.getName()).append(",");
            }else {
                String dictLabel = sysDictService.getDictLabel("RELATIONSHIP", buyer.getRelationship());
                builder.append(buyer.getName())
                        .append("(")
                        .append(dictLabel)
                        .append(")")
                        .append(",");
            }
        }
        if(builder.length() > 0){
            builder.deleteCharAt(builder.length()-1);
        }
        model.setBuyerName(builder.toString());
        model.setUpdatedAt(date);
        model.setUpdatedId(userId);
        return updateByPrimaryKeySelective(model);

    }




    /**
     * 删除
     * @param ids
     * @return
     */
    public int delete(String ids){
        String[] idLsit = ids.split(",");
        for(String id : idLsit){
            CustomerManagement customerManagement = selectByPrimaryKey(id);
            String houseId = customerManagement.getHouseId();
            if(StringUtils.isNotEmpty(houseId)){
                throw new RuntimeException("该数据已在别的业务中引用，无法删除!!!");
            }
        }
        for(String id : idLsit){
             deleteByPrimaryKey(id);
        }
        return 1;
    }

    /**
     * 查看详情
     * @param id
     * @return
     */
    public CustomerManagementDto getById(String id){
        CustomerManagement customerManagement = selectByPrimaryKey(id);
        CustomerManagementDto result = new CustomerManagementDto();
        BeanUtil.copyProperties(customerManagement,result);
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.setOrderByClause("order_num asc");
        buyerExample.createCriteria().andCustomerManagementIdEqualTo(id);
        List<Buyer> buyers = buyerService.selectByExample(buyerExample);
        if(CollectionUtils.isNotEmpty(buyers)){
            String json = JSON.toJSON(buyers).toString();
            result.setBuyerJson(json);
        }
        if(StringUtils.isNotEmpty(customerManagement.getAbandonmentId())){
            FrameUser frameUser = frameUserService.selectByPrimaryKey(customerManagement.getAbandonmentId());
            String name = frameUser == null ? "" : frameUser.getTruename();
            result.setAbandonmentName(name);
        }

        return result;
    }

    /**
     * 放弃资格
     * @param model
     * @return
     */
    public int giveUp(CustomerManagement model){

        model.setStatus(6);
        model.setAbandonmentAt(new Date());
        model.setAbandonmentId(ShiroUtils.getUserId());
        int result = updateByPrimaryKeySelective(model);

        RoomSelectionInfoExample example = new RoomSelectionInfoExample();
        example.createCriteria().andCustomerManagementIdEqualTo(model.getId());
        List<RoomSelectionInfo> list = roomSelectionInfoMapper.selectByExample(example);
        for(RoomSelectionInfo roomSelectionInfo : list){
            ParkRoom parkRoom = new ParkRoom();
            parkRoom.setId(roomSelectionInfo.getHouseId());
            parkRoom.setIsChoose(0);
            parkRoomMapper.updateByPrimaryKeySelective(parkRoom);
        }

        return result;
    }

    /**
     * 选房通知单下载
     * @param id
     * @param response
     */
    public void downLoad(String id, HttpServletResponse response) {

        String filePath = "/home/project/settleDown/tem/houseSelectionNotice.docx";
        String fileName = "选房通知单.docx";
        CustomerManagement customerManagement = selectByPrimaryKey(id);
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.setOrderByClause("order_num asc");
        buyerExample.createCriteria().andCustomerManagementIdEqualTo(id);
        List<Buyer> buyers = buyerService.selectByExample(buyerExample);
        if(CollectionUtils.isNotEmpty(buyers)){
            Buyer buyer = buyers.get(0);
            String name = buyer.getName();
            String identityCard = buyer.getIdentityCard();
            String roomSelectionNumber = customerManagement.getRoomSelectionNumber();
            Date roomSelectionDate = customerManagement.getRoomSelectionDate();
            String year = DateUtils.parseDateToStr(DateUtils.YYYY, roomSelectionDate);
            String month = DateUtils.parseDateToStr(DateUtils.MM, roomSelectionDate);
            String date = DateUtils.parseDateToStr(DateUtils.DD, roomSelectionDate);
            ParkInfo parkInfo = parkInfoMapper.selectByPrimaryKey(customerManagement.getGroupId());
            String region = "";
            String dictLabel = sysDictService.getDictLabel("REGIONALISM_CODE", parkInfo.getRegionalismCode());
            if(StringUtils.isNotEmpty(dictLabel)){
                region = dictLabel;
            }
            Map<String, Object> data = new HashMap<>();
            Map<String, Object> picData = new HashMap<>();
            Map<String, Object> sealData = new HashMap<>();
            data.put("${name}", name);
            data.put("${identityCard}", identityCard);
            data.put("${roomSelectionNumber}", roomSelectionNumber);
            data.put("${year}", year);
            data.put("${month}", month);
            data.put("${date}", date);
            data.put("${region}", region);
            ReplaceWord.operateWord(response,filePath,fileName,data, picData,sealData);
        }




    }




}
