package com.idea.customerManagement.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.mapper.ParkFloorDiscMapper;
import com.idea.buildManage.mapper.ParkInfoMapper;
import com.idea.buildManage.mapper.ParkRoomExtendMapper;
import com.idea.buildManage.mapper.ParkRoomMapper;
import com.idea.buildManage.model.ParkFloorDisc;
import com.idea.buildManage.model.ParkInfo;
import com.idea.buildManage.model.ParkRoom;
import com.idea.buildManage.response.ParkRoomResponse;
import com.idea.customerManagement.dto.ContractManageDto;
import com.idea.customerManagement.dto.IntentionalDepositDto;
import com.idea.customerManagement.dto.RoomSelectionInfoDto;
import com.idea.customerManagement.mapper.CustomerManagementMapper;
import com.idea.customerManagement.mapper.IntentionalDepositExtendMapper;
import com.idea.customerManagement.mapper.IntentionalDepositMapper;
import com.idea.customerManagement.mapper.RoomSelectionInfoMapper;
import com.idea.customerManagement.model.*;
import com.idea.invoice.model.ReceiptManage;
import com.idea.invoice.model.ReceiptManageExample;
import com.idea.invoice.service.ReceiptManageService;
import com.idea.paymentManagement.mapper.PayLogMapper;
import com.idea.paymentManagement.model.PayLog;
import com.idea.util.DateUtils;
import com.idea.util.MoneyUtils;
import com.idea.util.ReplaceWord;
import com.rockstar.common.base.BaseService;
import com.rockstar.frame.model.FrameUser;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.frame.service.FrameUserService;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.StringUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.util.*;

@Service
public class IntentionalDepositService implements BaseService<IntentionalDeposit, IntentionalDepositExample> {

    @Autowired
    private IntentionalDepositMapper modelMapper;
    @Autowired
    private CustomerManagementMapper customerManagementMapper;
    @Autowired
    private ParkInfoMapper parkInfoMapper;
    @Autowired
    private ParkFloorDiscMapper parkFloorDiscMapper;
    @Autowired
    private FrameUserService frameUserService;
    @Autowired
    private IntentionalDepositExtendMapper extendMapper;
    @Autowired
    private ParkRoomExtendMapper roomExtendMapper;
    @Autowired
    private RoomSelectionInfoMapper roomSelectionInfoMapper;
    @Autowired
    private ParkRoomMapper parkRoomMapper;
    @Autowired
    private ContractManageService contractManageService;
    @Autowired
    private PayLogMapper payLogMapper;
    @Autowired
    private ReceiptManageService receiptManageService;
    @Autowired
    private BuyerService buyerService;
    @Autowired
    private SysDictService sysDictService;


    @Override
    public int deleteByPrimaryKey(String s) {
        return modelMapper.deleteByPrimaryKey(s);
    }

    @Override
    public int insertSelective(IntentionalDeposit intentionalDeposit) {
        intentionalDeposit.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(intentionalDeposit);
    }

    public int insertWithoutId(IntentionalDeposit intentionalDeposit) {
        return modelMapper.insertSelective(intentionalDeposit);
    }

    @Override
    public IntentionalDeposit selectByPrimaryKey(String s) {
        return modelMapper.selectByPrimaryKey(s);
    }

    @Override
    public int updateByPrimaryKeySelective(IntentionalDeposit intentionalDeposit) {
        return modelMapper.updateByPrimaryKeySelective(intentionalDeposit);
    }

    @Override
    public int updateByExampleSelective(IntentionalDeposit intentionalDeposit, IntentionalDepositExample intentionalDepositExample) {
        return modelMapper.updateByExampleSelective(intentionalDeposit,intentionalDepositExample);
    }

    @Override
    public int updateByExample(IntentionalDeposit intentionalDeposit, IntentionalDepositExample intentionalDepositExample) {
        return modelMapper.updateByExample(intentionalDeposit,intentionalDepositExample);
    }

    @Override
    public List<IntentionalDeposit> selectByExample(IntentionalDepositExample intentionalDepositExample) {
        return modelMapper.selectByExample(intentionalDepositExample);
    }

    @Override
    public long countByExample(IntentionalDepositExample intentionalDepositExample) {
        return modelMapper.countByExample(intentionalDepositExample);
    }

    @Override
    public int deleteByExample(IntentionalDepositExample intentionalDepositExample) {
        return modelMapper.deleteByExample(intentionalDepositExample);
    }

    /**
     * 获取收据流水号（意向金编号）
     * @param customerManagementId
     * @return
     */
    public String getSerialNumber(String customerManagementId){
        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(customerManagementId);
        String groupId = customerManagement.getGroupId();
        String discId = customerManagement.getDiscId();
        // 查找这个小区已收取意向金的 数据量
        IntentionalDepositExample example = new IntentionalDepositExample();
        example.createCriteria().andGroupIdEqualTo(groupId);
        Long count = countByExample(example);
        String number = "";
        if(count.doubleValue() == 0){
            number = "00001";
        }else {
            count ++;
            number = String.format("%05d", count);
        }
        ParkInfo parkInfo = parkInfoMapper.selectByPrimaryKey(groupId);
        ParkFloorDisc parkFloorDisc = parkFloorDiscMapper.selectByPrimaryKey(discId);
        String serialNumber = parkInfo.getGroupName()+ parkFloorDisc.getName() + "-YXJ" + number;
        return serialNumber;
    }

    /**
     * 获取收据流水号（定金编号）
     * @param id
     * @return
     */
    public String getDepositSerialNumber(String id){

        IntentionalDeposit deposit = selectByPrimaryKey(id);
        String groupId = deposit.getGroupId();
        String discId = deposit.getDiscId();
        ParkInfo parkInfo = parkInfoMapper.selectByPrimaryKey(groupId);
        ParkFloorDisc parkFloorDisc = parkFloorDiscMapper.selectByPrimaryKey(discId);
        // 查找这个小区已收取认购金的 数据量
        RoomSelectionInfoExample example = new RoomSelectionInfoExample();
        example.createCriteria().andGroupIdEqualTo(groupId).andStatusEqualTo(2);
        Long count = roomSelectionInfoMapper.countByExample(example);
        String number = "";
        if(count.doubleValue() == 0){
            number = "00001";
        }else {
            count ++;
            number = String.format("%05d", count);
        }
        String serialNumber = parkInfo.getGroupName()+ parkFloorDisc.getName() + "-DJ" + number;
        return serialNumber;
    }


    /**
     * 意向金登记
     * @param model
     * @return
     */
    public int add(IntentionalDeposit model){

        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(model.getCustomerManagementId());
        String id = IdUtil.simpleUUID();
        // 已收取
        model.setStatus(1);
        model.setGroupId(customerManagement.getGroupId());
        model.setDiscId(customerManagement.getDiscId());
        model.setCreatedAt(new Date());
        model.setCreatedId(ShiroUtils.getUserId());
        model.setId(id);
        int result = insertWithoutId(model);
        // 票据管理 新增一条记录
        ReceiptManage receiptManage = new ReceiptManage();
        receiptManage.setBatchNumber(customerManagement.getBatchNumber());
        receiptManage.setRoomSelectionNumber(customerManagement.getRoomSelectionNumber());
        receiptManage.setBuyerName(customerManagement.getBuyerName());
        receiptManage.setCustomerManagementId(model.getCustomerManagementId());
        receiptManage.setBusinessId(id);
        // 票据类型 意向金
        receiptManage.setReceiptType("1");
        receiptManage.setReceiptMoney(model.getReceivedAmount());
        receiptManage.setReceiptNumber(model.getSerialNumber());
        receiptManageService.add(receiptManage);
        return result;
    }

    /**
     * 意向金修改
     * @param model
     * @return
     */
    public int edit(IntentionalDeposit model){

        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(model.getCustomerManagementId());
        model.setGroupId(customerManagement.getGroupId());
        model.setDiscId(customerManagement.getDiscId());
        model.setUpdatedAt(new Date());
        model.setUpdatedId(ShiroUtils.getUserId());
        return updateByPrimaryKeySelective(model);
    }

    /**
     * 根据客户id查找意向金
     * @param customerManagementId
     * @return
     */
    public IntentionalDepositDto getByCustomerManagementId(String customerManagementId){

        IntentionalDepositExample example = new IntentionalDepositExample();
        example.createCriteria().andCustomerManagementIdEqualTo(customerManagementId);
        List<IntentionalDeposit> intentionalDeposits = modelMapper.selectByExampleWithBLOBs(example);
        if(CollectionUtils.isNotEmpty(intentionalDeposits)){
            IntentionalDepositDto result = new IntentionalDepositDto();
            IntentionalDeposit intentionalDeposit = intentionalDeposits.get(0);
            BeanUtil.copyProperties(intentionalDeposit,result);
            FrameUser frameUser = frameUserService.selectByPrimaryKey(intentionalDeposit.getCreatedId());
            if(ObjectUtil.isNotNull(frameUser)){
                result.setCreatedName(frameUser.getTruename());
            }
            return result;
        }
        return null;

    }

    /**
     * 分页
     * @param tablepar
     * @param model
     * @param dt
     * @return
     */
    public PageInfo<IntentionalDepositDto> listByModel(Tablepar tablepar, IntentionalDepositDto model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<IntentionalDepositDto> list = extendMapper.listByModel(model);
        PageInfo<IntentionalDepositDto> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 意向金转定金
     * @param model
     * @return
     */
    @Transactional
    public int toDeposit(IntentionalDeposit model) {


        // 转定金
        model.setStatus(2);
        model.setHandleId(ShiroUtils.getUserId());
        model.setHandleDate(new Date());
        int result = modelMapper.updateByPrimaryKeySelective(model);

        String houseId = model.getHouseId();
        String customerManagementId = model.getCustomerManagementId();

        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(customerManagementId);
        // 购房状态 已选房
        customerManagement.setStatus(2);
        customerManagement.setHouseId(houseId);
        StringBuilder builder = new StringBuilder();

        // 拼接房屋名称
        ParkRoomResponse response = roomExtendMapper.getFullName(houseId);
        builder.append(response.getBuildName()).append("-").append(response.getRoomNo()).append(",");
        if (builder.length() > 0) {
            builder.deleteCharAt(builder.length() - 1);
        }
        customerManagement.setHouseName(builder.toString());
        customerManagementMapper.updateByPrimaryKeySelective(customerManagement);

        // 新增选房信息
        String roomSelectionInfoId = IdUtil.simpleUUID();
        RoomSelectionInfo roomSelectionInfo = new RoomSelectionInfo();
        roomSelectionInfo.setId(roomSelectionInfoId);
        roomSelectionInfo.setCustomerManagementId(customerManagementId);
        roomSelectionInfo.setHouseId(response.getId());
        roomSelectionInfo.setBuildId(response.getBuildId());
        roomSelectionInfo.setDiscId(response.getDiscId());
        roomSelectionInfo.setGroupId(response.getGroupId());
        roomSelectionInfo.setCreatedAt(new Date());
        roomSelectionInfo.setCreatedId(ShiroUtils.getUserId());
        roomSelectionInfo.setReceivableMoney(model.getReceivableMoney());
        roomSelectionInfo.setReceivedAmount(model.getReceivedAmount());
        // 收取状态 意向金转入
        roomSelectionInfo.setStatus(3);
        roomSelectionInfo.setSerialNumber(model.getDepositSerialNumber());
        roomSelectionInfo.setPaymentMethod(model.getPaymentMethod());
        roomSelectionInfo.setCollectionTime(model.getCollectionTime());
        roomSelectionInfoMapper.insertSelective(roomSelectionInfo);

        // 转定金后生产合同
        String contractId = IdUtil.simpleUUID();
        contractManageService.addContract(roomSelectionInfo,contractId);

        // 新增一条定金收款记录
        PayLog payLog = new PayLog();
        payLog.setId(IdUtil.simpleUUID());
        payLog.setHouseId(houseId);
        payLog.setContractId(contractId);
        payLog.setCustomerManagementId(customerManagementId);
        // 收款类型 预收款
        payLog.setPayType(1);
        // 款项内容 定金
        payLog.setContentType(1);
        payLog.setPayMoney(model.getReceivedAmount());
        payLog.setPayTime(new Date());
        payLog.setPaymentMethod(model.getPaymentMethod());
        payLog.setStatus(0);
        payLog.setCreatedAt(new Date());
        payLog.setCreatedBy(ShiroUtils.getUserId());
        payLogMapper.insertSelective(payLog);

        // 更新房间状态
        ParkRoom parkRoom = new ParkRoom();
        parkRoom.setId(response.getId());
        parkRoom.setIsChoose(1);
        // 已认购
        parkRoom.setSoldStatus(2);
        parkRoomMapper.updateByPrimaryKeySelective(parkRoom);

        // 票据管理 新增一条记录
        ReceiptManage receiptManage = new ReceiptManage();
        receiptManage.setBatchNumber(customerManagement.getBatchNumber());
        receiptManage.setRoomSelectionNumber(customerManagement.getRoomSelectionNumber());
        receiptManage.setBuyerName(customerManagement.getBuyerName());
        receiptManage.setCustomerManagementId(model.getCustomerManagementId());
        receiptManage.setBusinessId(roomSelectionInfoId);
        // 票据类型 定金
        receiptManage.setReceiptType("2");
        receiptManage.setReceiptMoney(model.getReceivedAmount());
        receiptManage.setReceiptNumber(model.getDepositSerialNumber());
        // 已开
        receiptManage.setReceiptStatus(1);
        receiptManageService.add(receiptManage);
        // 更新关联的意向金数据 更新意向金票据关联票号
        ReceiptManageExample receiptManageExample = new ReceiptManageExample();
        receiptManageExample.createCriteria().andBusinessIdEqualTo(model.getId());
        List<ReceiptManage> receiptManages = receiptManageService.selectByExample(receiptManageExample);
        if(CollectionUtils.isNotEmpty(receiptManages)){
            ReceiptManage manage = receiptManages.get(0);
            manage.setAssociationNumber(model.getDepositSerialNumber());
            // 已换票
            manage.setReceiptStatus(2);
            receiptManageService.updateByPrimaryKeySelective(manage);
        }



        return result;

    }

    /**
     * 退意向金
     * @param model
     * @return
     */
    public int refund(IntentionalDeposit model){

        // 退意向金
        model.setStatus(3);
        model.setHandleId(ShiroUtils.getUserId());
        model.setHandleDate(new Date());
        int result = modelMapper.updateByPrimaryKeySelective(model);
        return result;
    }

    public IntentionalDepositDto getById(String id) {
        IntentionalDeposit intentionalDeposit = modelMapper.selectByPrimaryKey(id);
        IntentionalDepositDto intentionalDepositDto = new IntentionalDepositDto();
        String handleName = "";
        if(StringUtils.isNotEmpty(intentionalDeposit.getHandleId())){
            FrameUser frameUser = frameUserService.selectByPrimaryKey(intentionalDeposit.getHandleId());
            handleName = frameUser == null ? "" : frameUser.getTruename();
        }
        BeanUtil.copyProperties(intentionalDeposit,intentionalDepositDto);
        intentionalDepositDto.setHandleName(handleName);
        return intentionalDepositDto;
    }

    /**
     * 意向金收据
     * @param model
     * @param response
     */
    public void downLoadReceipt(IntentionalDeposit model, HttpServletResponse response) {

        // TODO: 2024/10/24 意向金收据暂无word文档
        String filePath = "/home/project/settleDown/tem/.docx";
        String fileName = "意向金收据.docx";
        String customerManagementId = model.getCustomerManagementId();
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.setOrderByClause("order_num asc");
        buyerExample.createCriteria().andCustomerManagementIdEqualTo(customerManagementId);
        List<Buyer> buyers = buyerService.selectByExample(buyerExample);
        String name = "";
        if (CollectionUtils.isNotEmpty(buyers)) {
            Buyer buyer = buyers.get(0);
            name = buyer.getName();
        }
        String collectionTime = DateUtils.parseDateToStr(DateUtils.DATE_CH, model.getCollectionTime());
        String content = "定金";
        String paymentMethod = "";
        String dictLabel = sysDictService.getDictLabel("COLLECTION_METHODS", model.getPaymentMethod().toString());
        if(StringUtils.isNotEmpty(dictLabel)){
            paymentMethod = dictLabel;
        }
        String receivedAmount = model.getReceivedAmount().toString();
        String capital = MoneyUtils.amountConversion(receivedAmount);

        Map<String, Object> data = new HashMap<>();
        Map<String, Object> picData = new HashMap<>();
        Map<String, Object> sealData = new HashMap<>();
        data.put("${name}", name);
        data.put("${collectionTime}", collectionTime);
        data.put("${content}", content);
        data.put("${paymentMethod}", paymentMethod);
        data.put("${capital}", capital);
        data.put("${receivedAmount}", receivedAmount);

        ReplaceWord.operateWord(response, filePath, fileName, data, picData, sealData);


    }



}
