package com.idea.customerManagement.service;

import cn.hutool.core.math.Money;
import cn.hutool.core.util.IdUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.mapper.MnpBuildingMapper;
import com.idea.buildManage.mapper.ParkRoomExtendMapper;
import com.idea.buildManage.mapper.ParkRoomMapper;
import com.idea.buildManage.model.ParkRoom;
import com.idea.buildManage.response.ParkRoomResponse;
import com.idea.customerManagement.dto.ContractManageDto;
import com.idea.customerManagement.dto.RoomSelectionInfoDto;
import com.idea.customerManagement.mapper.CustomerManagementMapper;
import com.idea.customerManagement.mapper.RoomSelectionInfoExtendMapper;
import com.idea.customerManagement.mapper.RoomSelectionInfoMapper;
import com.idea.customerManagement.model.*;
import com.idea.invoice.model.ReceiptManage;
import com.idea.invoice.service.ReceiptManageService;
import com.idea.paymentManagement.mapper.PayLogMapper;
import com.idea.paymentManagement.model.PayLog;
import com.idea.util.DateUtils;
import com.idea.util.MoneyUtils;
import com.idea.util.ReplaceWord;
import com.rockstar.common.base.BaseService;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.StringUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.*;

@Service
public class RoomSelectionInfoService implements BaseService<RoomSelectionInfo, RoomSelectionInfoExample> {

    @Autowired
    private RoomSelectionInfoMapper modelMapper;
    @Autowired
    private RoomSelectionInfoExtendMapper extendMapper;
    @Autowired
    private CustomerManagementMapper customerManagementMapper;
    @Autowired
    private ParkRoomExtendMapper roomExtendMapper;
    @Autowired
    private ContractManageService contractManageService;
    @Autowired
    private BuyerService buyerService;
    @Autowired
    private ParkRoomMapper parkRoomMapper;
    @Autowired
    private PayLogMapper payLogMapper;
    @Autowired
    private ReceiptManageService receiptManageService;
    @Autowired
    private SysDictService sysDictService;


    @Override
    public int deleteByPrimaryKey(String s) {
        return modelMapper.deleteByPrimaryKey(s);
    }

    @Override
    public int insertSelective(RoomSelectionInfo roomSelectionInfo) {
        roomSelectionInfo.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(roomSelectionInfo);
    }

    public int insertWithoutId(RoomSelectionInfo roomSelectionInfo) {
        return modelMapper.insertSelective(roomSelectionInfo);
    }

    @Override
    public RoomSelectionInfo selectByPrimaryKey(String s) {
        return modelMapper.selectByPrimaryKey(s);
    }

    @Override
    public int updateByPrimaryKeySelective(RoomSelectionInfo roomSelectionInfo) {
        return modelMapper.updateByPrimaryKeySelective(roomSelectionInfo);
    }

    @Override
    public int updateByExampleSelective(RoomSelectionInfo roomSelectionInfo, RoomSelectionInfoExample roomSelectionInfoExample) {
        return modelMapper.updateByExampleSelective(roomSelectionInfo, roomSelectionInfoExample);
    }

    @Override
    public int updateByExample(RoomSelectionInfo roomSelectionInfo, RoomSelectionInfoExample roomSelectionInfoExample) {
        return modelMapper.updateByExample(roomSelectionInfo, roomSelectionInfoExample);
    }

    @Override
    public List<RoomSelectionInfo> selectByExample(RoomSelectionInfoExample roomSelectionInfoExample) {
        return modelMapper.selectByExample(roomSelectionInfoExample);
    }

    @Override
    public long countByExample(RoomSelectionInfoExample roomSelectionInfoExample) {
        return modelMapper.countByExample(roomSelectionInfoExample);
    }

    @Override
    public int deleteByExample(RoomSelectionInfoExample roomSelectionInfoExample) {
        return modelMapper.deleteByExample(roomSelectionInfoExample);
    }

    /**
     * 选房信息列表
     *
     * @param customerManagementId
     * @return
     */
    public List<RoomSelectionInfoDto> listAll(String customerManagementId) {
        List<RoomSelectionInfoDto> result = extendMapper.selectHouseListByCustomerId(customerManagementId);
        return result;
    }

    /**
     * 提交选房
     *
     * @param houseIds
     * @param customerManagementId
     * @return
     */
    public int submit(String houseIds, String customerManagementId) {

        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(customerManagementId);
        customerManagement.setStatus(2);
        customerManagement.setHouseId(houseIds);
        String[] split = houseIds.split(",");
        StringBuilder builder = new StringBuilder();
        // 先删除
        RoomSelectionInfoExample example = new RoomSelectionInfoExample();
        example.createCriteria().andCustomerManagementIdEqualTo(customerManagementId);
        List<RoomSelectionInfo> list = selectByExample(example);
        if (CollectionUtils.isNotEmpty(list)) {
            for (RoomSelectionInfo info : list) {
                deleteByPrimaryKey(info.getId());
                // 还原房间选房状态
                ParkRoom parkRoom = new ParkRoom();
                parkRoom.setId(info.getHouseId());
                parkRoom.setIsChoose(0);
                parkRoomMapper.updateByPrimaryKeySelective(parkRoom);
            }
        }
        // 再新增
        for (String s : split) {
            // 拼接房屋名称
            ParkRoomResponse response = roomExtendMapper.getFullName(s);
            builder.append(response.getBuildName()).append("-").append(response.getRoomNo()).append(",");
            // 新增选房信息
            RoomSelectionInfo roomSelectionInfo = new RoomSelectionInfo();
            roomSelectionInfo.setCustomerManagementId(customerManagementId);
            roomSelectionInfo.setHouseId(response.getId());
            roomSelectionInfo.setBuildId(response.getBuildId());
            roomSelectionInfo.setDiscId(response.getDiscId());
            roomSelectionInfo.setGroupId(response.getGroupId());
            roomSelectionInfo.setCreatedAt(new Date());
            roomSelectionInfo.setCreatedId(ShiroUtils.getUserId());
            roomSelectionInfo.setReceivableMoney(response.getSubscriptionFunds());
            roomSelectionInfo.setStatus(1);
            insertSelective(roomSelectionInfo);

            ParkRoom parkRoom = new ParkRoom();
            parkRoom.setId(response.getId());
            parkRoom.setIsChoose(1);
            parkRoomMapper.updateByPrimaryKeySelective(parkRoom);

        }
        if (builder.length() > 0) {
            builder.deleteCharAt(builder.length() - 1);
        }
        customerManagement.setHouseName(builder.toString());
        customerManagementMapper.updateByPrimaryKeySelective(customerManagement);
        return 1;
    }

    /**
     * 获取收据流水号（定金编号）
     *
     * @param houseId
     * @return
     */
    public String getSerialNumber(String houseId) {
        ParkRoomResponse response = roomExtendMapper.getFullName(houseId);
        String groupId = response.getGroupId();
        // 查找这个小区已收取认购金的 数据量
        RoomSelectionInfoExample example = new RoomSelectionInfoExample();
        example.createCriteria().andGroupIdEqualTo(groupId).andStatusEqualTo(2);
        Long count = countByExample(example);
        String number = "";
        if (count.doubleValue() == 0) {
            number = "00001";
        } else {
            count++;
            number = String.format("%05d", count);
        }
        String serialNumber = response.getGroupName() + response.getDiscName() + "-DJ" + number;
        return serialNumber;
    }

    /**
     * 认购金登记
     *
     * @param model
     * @return
     */
    @Transactional
    public int add(RoomSelectionInfoDto model) {

        model.setUpdatedAt(new Date());
        model.setUpdatedId(ShiroUtils.getUserId());
        model.setStatus(2);
        model.setCollectionId(ShiroUtils.getUserId());
        model.setCollectionTime(new Date());
        int result = updateByPrimaryKeySelective(model);
        // 生成合同
        String contractId = IdUtil.simpleUUID();
        contractManageService.addContract(model, contractId);
        // 修改房间已收状态 已认购
        ParkRoom parkRoom = new ParkRoom();
        parkRoom.setId(model.getHouseId());
        parkRoom.setSoldStatus(2);
        parkRoomMapper.updateByPrimaryKeySelective(parkRoom);
        // 新增一条定金收款记录
        PayLog payLog = new PayLog();
        payLog.setId(IdUtil.simpleUUID());
        payLog.setHouseId(model.getHouseId());
        payLog.setContractId(contractId);
        payLog.setCustomerManagementId(model.getCustomerManagementId());
        // 收款类型 预收款
        payLog.setPayType(1);
        // 款项内容 定金
        payLog.setContentType(1);
        payLog.setPayMoney(model.getReceivedAmount());
        payLog.setPayTime(new Date());
        payLog.setPaymentMethod(model.getPaymentMethod());
        payLog.setStatus(0);
        payLog.setCreatedAt(new Date());
        payLog.setCreatedBy(ShiroUtils.getUserId());
        payLogMapper.insertSelective(payLog);
        // 票据管理 新增一条记录
        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(model.getCustomerManagementId());
        ReceiptManage receiptManage = new ReceiptManage();
        receiptManage.setBatchNumber(customerManagement.getBatchNumber());
        receiptManage.setRoomSelectionNumber(customerManagement.getRoomSelectionNumber());
        receiptManage.setBuyerName(customerManagement.getBuyerName());
        receiptManage.setCustomerManagementId(model.getCustomerManagementId());
        receiptManage.setBusinessId(model.getId());
        // 票据类型 定金
        receiptManage.setReceiptType("2");
        receiptManage.setReceiptMoney(model.getReceivedAmount());
        receiptManage.setReceiptNumber(model.getSerialNumber());
        receiptManageService.add(receiptManage);
        return result;
    }

    /**
     * 认购金修改
     *
     * @param model
     * @return
     */
    public int edit(RoomSelectionInfoDto model) {

        model.setUpdatedAt(new Date());
        model.setUpdatedId(ShiroUtils.getUserId());
        model.setCollectionId(ShiroUtils.getUserId());
        return updateByPrimaryKeySelective(model);
    }

    /**
     * 查看详情
     *
     * @param id
     * @return
     */
    public RoomSelectionInfo getById(String id) {
        return selectByPrimaryKey(id);
    }

    /**
     * 房源确认单 下载
     *
     * @param model
     * @param request
     * @param response
     */
    public void downLoad(RoomSelectionInfoDto model, HttpServletRequest request, HttpServletResponse response) {

        String filePath = "/home/project/settleDown/tem/roomConfirmationSheet.docx";
        String picturePath = "C:\\Users\\35838\\Pictures\\liuhaizhu.jpg";
        String sealPath = "C:\\Users\\35838\\Pictures\\seal.png";
        String fileName = "房源确认单.docx";
        String customerManagementId = model.getCustomerManagementId();
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.setOrderByClause("order_num asc");
        buyerExample.createCriteria().andCustomerManagementIdEqualTo(customerManagementId);
        List<Buyer> buyers = buyerService.selectByExample(buyerExample);
        if (CollectionUtils.isNotEmpty(buyers)) {
            Buyer buyer = buyers.get(0);
            ParkRoomResponse roomResponse = roomExtendMapper.getFullName(model.getHouseId());
            String fullName = roomResponse.getGroupName() + roomResponse.getDiscName() + roomResponse.getBuildName()
                    + roomResponse.getRoomNo();
            String loginName = ShiroUtils.getLoginName();
            String roomChooseDate = DateUtils.parseDateToStr(DateUtils.DATE_CH, new Date());

            Map<String, Object> data = new HashMap<>();
            Map<String, Object> picData = new HashMap<>();
            Map<String, Object> sealData = new HashMap<>();
            data.put("${name}", buyer.getName());
            data.put("${identityCard}", buyer.getIdentityCard());
            data.put("${address}", fullName);
            data.put("${area}", roomResponse.getPredictionBuildArea());
            data.put("${roomNumber}", roomResponse.getRoomNumber());
            data.put("${roomChooseDate}", roomChooseDate);
            data.put("${username}", loginName);
            picData.put("${picture}", picturePath);
            sealData.put("${seal}", sealPath);
            ReplaceWord.operateWord(response, filePath, fileName, data, picData, sealData);
        }


    }


    /**
     * 定金列表
     *
     * @param tablepar
     * @param model
     * @param dt
     * @return
     */
    public PageInfo<RoomSelectionInfoDto> listByModel(Tablepar tablepar, RoomSelectionInfoDto model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<RoomSelectionInfoDto> list = extendMapper.listByModel(model);
        PageInfo<RoomSelectionInfoDto> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 定金收据
     * @param model
     * @param response
     */
    public void downLoadReceipt(RoomSelectionInfoDto model, HttpServletResponse response) {

        // TODO: 2024/10/24 定金收据暂无word文档
        String filePath = "/home/project/settleDown/tem/.docx";
        String fileName = "定金收据.docx";
        String customerManagementId = model.getCustomerManagementId();
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.setOrderByClause("order_num asc");
        buyerExample.createCriteria().andCustomerManagementIdEqualTo(customerManagementId);
        List<Buyer> buyers = buyerService.selectByExample(buyerExample);
        String name = "";
        if (CollectionUtils.isNotEmpty(buyers)) {
            Buyer buyer = buyers.get(0);
            name = buyer.getName();
        }
        ParkRoomResponse roomResponse = roomExtendMapper.getFullName(model.getHouseId());
        String fullName = roomResponse.getGroupName()
                + "-"
                + roomResponse.getDiscName()
                + roomResponse.getBuildName()
                + roomResponse.getRoomNo();

        String collectionTime = DateUtils.parseDateToStr(DateUtils.DATE_CH, model.getCollectionTime());
        String content = "定金";
        String paymentMethod = "";
        String dictLabel = sysDictService.getDictLabel("COLLECTION_METHODS", model.getPaymentMethod().toString());
        if(StringUtils.isNotEmpty(dictLabel)){
            paymentMethod = dictLabel;
        }
        String receivedAmount = model.getReceivedAmount().toString();
        String capital = MoneyUtils.amountConversion(receivedAmount);

        Map<String, Object> data = new HashMap<>();
        Map<String, Object> picData = new HashMap<>();
        Map<String, Object> sealData = new HashMap<>();
        data.put("${name}", name);
        data.put("${collectionTime}", collectionTime);
        data.put("${content}", content);
        data.put("${paymentMethod}", paymentMethod);
        data.put("${fullName}", fullName);
        data.put("${capital}", capital);
        data.put("${receivedAmount}", receivedAmount);

        ReplaceWord.operateWord(response, filePath, fileName, data, picData, sealData);


    }


}
