package com.idea.invoice.controller;

import com.github.pagehelper.PageInfo;
import com.idea.invoice.dto.InvoiceManageDto;
import com.idea.invoice.model.InvoiceBank;
import com.idea.invoice.model.InvoiceManage;
import com.idea.invoice.service.InvoiceManageService;
import com.rockstar.common.base.BaseController;
import com.rockstar.common.domain.AjaxResult;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.TableSplitResult;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.util.StringUtils;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.List;

@Controller
@RequestMapping(value = "InvoiceManageController")
@Api(value="开票管理")
public class InvoiceManageController extends BaseController {

    @Autowired
    private InvoiceManageService modelService;

    /**
     * 预收款发票
     * @param tablepar
     * @param model
     * @param dt
     * @return
     */
    @PostMapping(value = "advancePaymentInvoiceList",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object advancePaymentInvoiceList(Tablepar tablepar, InvoiceManageDto model, DateTrans dt){
        PageInfo<InvoiceManageDto> page= modelService.advancePaymentInvoiceList(tablepar,model,dt);
        return new TableSplitResult<>(page.getPageNum(), page.getTotal(), page.getList());
    }

    /**
     * 正式发票
     * @param tablepar
     * @param model
     * @param dt
     * @return
     */
    @PostMapping(value = "formalInvoiceList",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object formalInvoiceList(Tablepar tablepar, InvoiceManageDto model, DateTrans dt){
        PageInfo<InvoiceManageDto> page= modelService.formalInvoiceList(tablepar,model,dt);
        return new TableSplitResult<>(page.getPageNum(), page.getTotal(), page.getList());
    }

    /**
     * 新增预收款发票
     * @param record
     * @return
     */
    @PostMapping(value = "add",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult add(InvoiceManageDto record){
        int result = modelService.add(record);
        return result(result);
    }

    @PostMapping(value = "getById",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object getById(String id){
        InvoiceManageDto result = modelService.getById(id);
        return result;
    }


    @PostMapping(value = "getByContractId",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object getInfoByContract(String id){
        InvoiceManageDto result = modelService.getInfoByContract(id);
        return result;
    }

    /**
     * 查询对应的银行
     * @param id
     * @return
     */
    @PostMapping(value = "getBankTaxRateByFloor",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public Object getBankTaxRateByFloor(String id) {
        if(!StringUtils.isNotEmpty(id)){
            return AjaxResult.error(503, "参数不能为空");
        }
        return modelService.getBankTaxRateByFloor(id);
    }

    @PostMapping(value = "batchListVo",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public List<InvoiceManageDto> batchListVo(String invoiceManages){
        return modelService.batchListVo(invoiceManages);
    }


    /**
     * 推送开票
     * @param record
     * @return
     */
    @PostMapping(value = "sendInvoice",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult sendInvoice(InvoiceManageDto record){
        int result = modelService.sendInvoice(record);
        return result(result);
    }

    /**
     * 批量推送开票
     * @param list
     * @return
     */
    @PostMapping(value = "batchSendInvoice",produces = {"application/json;charset=UTF-8"})
    @ResponseBody
    public AjaxResult batchSendInvoice(List<InvoiceManageDto> list){
        return  modelService.batchSendInvoice(list);
    }

}
