package com.idea.invoice.service;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.model.ParkFloorDisc;
import com.idea.buildManage.model.ParkFloorDiscExample;
import com.idea.buildManage.model.ParkInfo;
import com.idea.buildManage.model.ParkInfoExample;
import com.idea.buildManage.service.ParkFloorDiscService;
import com.idea.buildManage.service.ParkInfoService;
import com.idea.invoice.mapper.InvoiceBankMapper;
import com.idea.invoice.model.InvoiceBank;
import com.idea.invoice.model.InvoiceBankExample;
import com.rockstar.common.base.BaseService;
import com.rockstar.common.support.Convert;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.StringUtils;
import com.rockstar.util.UUIDUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class InvoiceBankService implements BaseService<InvoiceBank, InvoiceBankExample> {

    //文件mapper
    @Autowired
    private InvoiceBankMapper modelMapper;
    @Autowired
    private ParkInfoService parkInfoService;
    @Autowired
    private ParkFloorDiscService parkFloorDiscService;
    @Autowired
    private SysDictService sysDictService;

    public PageInfo<InvoiceBank> list(Tablepar tablepar, InvoiceBank model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<InvoiceBank> list = modelMapper.selectByExample(getCondition(model,dt));
        setData(list);
        return new PageInfo<>(list);
    }
    public List<InvoiceBank> listAll(InvoiceBank model, DateTrans dt) {
        List<InvoiceBank> list = modelMapper.selectByExample(getCondition(model,dt));
        setData(list);
        return list;
    }
    private void setData(List<InvoiceBank> list) {
        if(CollectionUtils.isNotEmpty(list)) {
            for (InvoiceBank bank : list) {
                if(StringUtils.isNotEmpty(bank.getSuitFloor())){
                    List<String> ids = Arrays.asList(bank.getSuitFloor().split(","));
                    ParkFloorDiscExample example = new ParkFloorDiscExample();
                    example.createCriteria().andIdIn(ids);
                    List<ParkFloorDisc> parkFloorDiscList = parkFloorDiscService.selectByExample(example);
                    if(CollectionUtils.isNotEmpty(parkFloorDiscList)){
                        List<String> groupIds = parkFloorDiscList.stream().map(ParkFloorDisc::getGroupId).distinct().collect(Collectors.toList());
                        if(CollectionUtils.isNotEmpty(groupIds)){
                            String resultStr = "";
                            for (String groupId : groupIds) {
                                ParkInfo parkInfo = parkInfoService.selectByPrimaryKey(groupId);
                                String str = parkInfo.getGroupName() + "-" + parkFloorDiscList.stream().filter(x -> groupId.equals(x.getGroupId()))
                                        .map(ParkFloorDisc::getName).collect(Collectors.joining(","));
                                if(StringUtils.isNotEmpty(resultStr)){
                                    resultStr = resultStr + "；" + str;
                                } else {
                                    resultStr = str;
                                }
                            }
                            bank.setSuitFloorStr(resultStr);
                        }
                    }
                }
                if(StringUtils.isNotEmpty(bank.getSuitCostType())) {
                    String[] suitCostType = bank.getSuitCostType().split(",");
                    String[] taxRate = bank.getTaxRate().split(",");
                    StringBuilder builder = new StringBuilder();
                    for(int i = 0 ; i < suitCostType.length; i++){
                        builder.append(suitCostType[i])
                                .append("(")
                                .append(Double.parseDouble(taxRate[i])*100).append("%")
                                .append(")")
                                .append(",");
                    }
                    if(builder.length() > 0){
                        builder.deleteCharAt(builder.length() - 1);
                    }
                    bank.setSuitCostTypeStr(builder.toString());

                }
            }
        }
    }

    private InvoiceBankExample getCondition(InvoiceBank model, DateTrans dt) {
        InvoiceBankExample ex = new InvoiceBankExample();
        InvoiceBankExample.Criteria criteria = ex.createCriteria();
        return ex;
    }

    @Override
    public int deleteByPrimaryKey(String ids) {
        List<String> list = Convert.toListStrArray(ids);
        InvoiceBankExample example = new InvoiceBankExample();
        example.createCriteria().andIdIn(list);
        return modelMapper.deleteByExample(example);
    }

    @Override
    public InvoiceBank selectByPrimaryKey(String id) {
        return modelMapper.selectByPrimaryKey(id);
    }

    @Override
    public int updateByPrimaryKeySelective(InvoiceBank record) {
        record.setUpdateTime(new Date());
        return modelMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int updateByExampleSelective(InvoiceBank record, InvoiceBankExample example) {
        record.setUpdateTime(new Date());
        return modelMapper.updateByExampleSelective(record, example);
    }

    @Override
    public int updateByExample(InvoiceBank record, InvoiceBankExample example) {
        record.setUpdateTime(new Date());
        return modelMapper.updateByExample(record, example);
    }

    @Override
    public List<InvoiceBank> selectByExample(InvoiceBankExample example) {
        return modelMapper.selectByExample(example);
    }

    @Override
    public long countByExample(InvoiceBankExample example) {
        return modelMapper.countByExample(example);
    }

    public long countByExample(InvoiceBank model, DateTrans dt) {
        return modelMapper.countByExample(getCondition(model,dt));
    }

    @Override
    public int deleteByExample(InvoiceBankExample example) {
        return modelMapper.deleteByExample(example);
    }

    public int insert(InvoiceBank record){
        record.setId(UUIDUtils.middleUUID());
        return insertWithoutId(record);
    }

    public int insertWithoutId(InvoiceBank record){
        record.setCreateTime(new Date());
        record.setUpdateTime(new Date());
        return modelMapper.insert(record);
    }

    @Override
    public int insertSelective(InvoiceBank record) {
        record.setId(UUIDUtils.middleUUID());
        record.setCreateTime(new Date());
        record.setUpdateTime(new Date());
        return modelMapper.insertSelective(record);
    }

    public JSONArray getTreeData() {
        JSONArray jsonArray = new JSONArray();
        List<ParkInfo> parkInfoList = parkInfoService.selectByExample(new ParkInfoExample());
        if(CollectionUtils.isNotEmpty(parkInfoList)){
            for (ParkInfo info : parkInfoList) {
                JSONObject jsonObject = new JSONObject();
                JSONArray jsonArray1 = new JSONArray();
                ParkFloorDiscExample example = new ParkFloorDiscExample();
                example.createCriteria().andGroupIdEqualTo(info.getId());
                List<ParkFloorDisc> floorList = parkFloorDiscService.selectByExample(example);
                if(CollectionUtils.isNotEmpty(floorList)){
                    jsonObject.put("id", info.getId());
                    jsonObject.put("label", info.getGroupName());
                    for (ParkFloorDisc floor : floorList) {
                        JSONObject jsonObject1 = new JSONObject();
                        jsonObject1.put("id", floor.getId());
                        jsonObject1.put("label", floor.getName());
                        jsonArray1.add(jsonObject1);
                    }
                    jsonObject.put("children", jsonArray1);
                    jsonArray.add(jsonObject);
                }
            }
        }
        return jsonArray;
    }

    /**
     * 查询适用费用类型
     * @param id
     * @return
     */
    public JSONArray getBillType(String id) {

        // 账单类型
        List<Map<String, String>> costType = sysDictService.selectDictList("COST_TYPE");
        JSONArray billJsonArray = new JSONArray();
        for (Map<String, String> stringStringMap : costType) {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("suitCostType",stringStringMap.get("label"));
            billJsonArray.add(jsonObject);
        }
        InvoiceBank invoiceBank = selectByPrimaryKey(id);
        String suitCostType = "";
        if(ObjectUtil.isNotNull(invoiceBank) && StringUtils.isNotEmpty(invoiceBank.getSuitCostType())){
            suitCostType = invoiceBank.getSuitCostType();
        }
        String taxRate = "";
        if(ObjectUtil.isNotNull(invoiceBank) && StringUtils.isNotEmpty(invoiceBank.getTaxRate())){
            taxRate = invoiceBank.getTaxRate();
        }
        String[] suitCostTypeSplit = suitCostType.split(",");
        String[] taxRateSplit = taxRate.split(",");
        JSONArray jsonArray = new JSONArray();
        for(int i = 0; i < suitCostTypeSplit.length; i ++) {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("suitCostType",suitCostTypeSplit[i]);
            jsonObject.put("taxRate",taxRateSplit[i]);
            jsonArray.add(jsonObject);
        }

        for(int i = 0; i < billJsonArray.size(); i++){
            JSONObject billJsonObject = billJsonArray.getJSONObject(i);
            for(int j = 0; j < jsonArray.size(); j++){
                JSONObject jsonObject = jsonArray.getJSONObject(j);
                //处理jsonObject
                if(billJsonObject.get("suitCostType").equals(jsonObject.get("suitCostType"))){
                    billJsonObject.put("taxRate",jsonObject.get("taxRate"));
                    billJsonObject.put("id", IdUtil.simpleUUID());
                }
            }
        }
        return billJsonArray;

    }


    public int edit(InvoiceBank record) {

        String infos = record.getInfos();
        List<InvoiceBank> invoiceBanks = JSONObject.parseArray(infos, InvoiceBank.class);
        StringBuilder suitCostType = new StringBuilder();
        StringBuilder taxRate = new StringBuilder();

        for(InvoiceBank bank: invoiceBanks){
            suitCostType.append(bank.getSuitCostType()).append(",");
            taxRate.append(bank.getTaxRate()).append(",");
        }
        if(suitCostType.length() > 0){
            suitCostType.deleteCharAt(suitCostType.length() - 1);
        }
        if(taxRate.length() > 0){
            taxRate.deleteCharAt(taxRate.length() - 1);
        }
        record.setSuitCostType(suitCostType.toString());
        record.setTaxRate(taxRate.toString());
        int result = updateByPrimaryKeySelective(record);
        return result;
    }


    public int add(InvoiceBank record) {

        String infos = record.getInfos();
        List<InvoiceBank> invoiceBanks = JSONObject.parseArray(infos, InvoiceBank.class);
        StringBuilder suitCostType = new StringBuilder();
        StringBuilder taxRate = new StringBuilder();

        for(InvoiceBank bank: invoiceBanks){
            suitCostType.append(bank.getSuitCostType()).append(",");
            taxRate.append(bank.getTaxRate()).append(",");
        }
        if(suitCostType.length() > 0){
            suitCostType.deleteCharAt(suitCostType.length() - 1);
        }
        if(taxRate.length() > 0){
            taxRate.deleteCharAt(taxRate.length() - 1);
        }
        record.setSuitCostType(suitCostType.toString());
        record.setTaxRate(taxRate.toString());
        int result = insertSelective(record);
        return result;
    }

}
