package com.idea.paymentManagement.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.mapper.ParkRoomExtendMapper;
import com.idea.buildManage.model.ParkInfo;
import com.idea.buildManage.model.ParkInfoExample;
import com.idea.buildManage.model.ParkRoom;
import com.idea.buildManage.response.ParkRoomResponse;
import com.idea.customerManagement.dto.ContractManageDto;
import com.idea.customerManagement.mapper.ContractManageMapper;
import com.idea.customerManagement.mapper.CustomerManagementMapper;
import com.idea.customerManagement.model.*;
import com.idea.customerManagement.service.BuyerService;
import com.idea.invoice.dto.InvoiceManageDto;
import com.idea.invoice.model.InvoiceManage;
import com.idea.invoice.model.InvoiceManageExample;
import com.idea.invoice.model.ReceiptManage;
import com.idea.invoice.service.InvoiceManageService;
import com.idea.invoice.service.ReceiptManageService;
import com.idea.paymentManagement.dto.PayLogDto;
import com.idea.paymentManagement.mapper.PayLogExtendMapper;
import com.idea.paymentManagement.mapper.PayLogMapper;
import com.idea.paymentManagement.model.PayLog;
import com.idea.paymentManagement.model.PayLogExample;
import com.idea.util.DateUtils;
import com.idea.util.MoneyUtils;
import com.idea.util.ReplaceWord;
import com.rockstar.common.base.BaseService;
import com.rockstar.frame.model.FrameUser;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.frame.service.FrameUserService;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.StringUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class PayLogService implements BaseService<PayLog, PayLogExample> {

    @Autowired
    private PayLogMapper modelMapper;
    @Autowired
    private PayLogExtendMapper extendMapper;
    @Autowired
    private ContractManageMapper contractManageMapper;
    @Autowired
    private FrameUserService frameUserService;
    @Autowired
    private InvoiceManageService invoiceManageService;
    @Autowired
    private CustomerManagementMapper customerManagementMapper;
    @Autowired
    private ReceiptManageService receiptManageService;
    @Autowired
    private BuyerService buyerService;
    @Autowired
    private SysDictService sysDictService;
    @Autowired
    private ParkRoomExtendMapper roomExtendMapper;


    @Override
    public int deleteByPrimaryKey(String s) {
        return modelMapper.deleteByPrimaryKey(s);
    }

    @Override
    public int insertSelective(PayLog PayLog) {
        PayLog.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(PayLog);
    }

    public int insertWithoutId(PayLog PayLog) {
        return modelMapper.insertSelective(PayLog);
    }

    @Override
    public PayLog selectByPrimaryKey(String s) {
        return modelMapper.selectByPrimaryKey(s);
    }

    @Override
    public int updateByPrimaryKeySelective(PayLog PayLog) {
        return modelMapper.updateByPrimaryKeySelective(PayLog);
    }

    @Override
    public int updateByExampleSelective(PayLog PayLog, PayLogExample PayLogExample) {
        return modelMapper.updateByExampleSelective(PayLog, PayLogExample);
    }

    @Override
    public int updateByExample(PayLog PayLog, PayLogExample PayLogExample) {
        return 0;
    }

    @Override
    public List<PayLog> selectByExample(PayLogExample PayLogExample) {
        return null;
    }

    @Override
    public long countByExample(PayLogExample PayLogExample) {
        return 0;
    }

    @Override
    public int deleteByExample(PayLogExample PayLogExample) {
        return 0;
    }


    public PageInfo<ContractManageDto> listByModel(Tablepar tablepar, ContractManageDto model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<ContractManageDto> list = extendMapper.listByModel(model);
        PageInfo<ContractManageDto> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }


    /**
     * 新增
     *
     * @param model
     * @return
     */
    @Transactional
    public int add(PayLog model) {

        String id = IdUtil.simpleUUID();
        model.setCreatedBy(ShiroUtils.getUserId());
        model.setCreatedAt(new Date());
        model.setStatus(0);
        model.setId(id);
        int result = insertWithoutId(model);
        // 更新合同 预收款 收款状态
        ContractManage contractManage = contractManageMapper.selectByPrimaryKey(model.getContractId());
        if (model.getPayType() == 1) {
            String receivedMoneyStr = extendMapper.getMoneyByContractId(model.getContractId(), model.getPayType());
            BigDecimal totalPrice = contractManage.getTotalPrice();
            BigDecimal receivedMoney = new BigDecimal(receivedMoneyStr);
            if (receivedMoney.doubleValue() < totalPrice.doubleValue()) {
                // 部分收款
                contractManage.setCollectionStatus(1);
            } else {
                // 全部收款
                contractManage.setCollectionStatus(2);

                // 查询合同的专项维修基础 收款情况
                Integer fundCollectionStatus = contractManage.getFundCollectionStatus();
                // 完全收款
                if(fundCollectionStatus == 2){
                    // 查询该合同是否已生成 正式发票
                    InvoiceManageExample invoiceManageExample = new InvoiceManageExample();
                    invoiceManageExample.createCriteria().andTypeEqualTo("1").andContractIdEqualTo(model.getContractId());
                    List<InvoiceManage> invoiceManages = invoiceManageService.selectByExample(invoiceManageExample);
                    if(CollectionUtils.isEmpty(invoiceManages)){
                        // 新增待推送的开票信息
                        InvoiceManageDto invoiceManage = invoiceManageService.getInfoByContract(model.getContractId());
                        // 正式发票
                        invoiceManage.setType("2");
                        // 房款
                        invoiceManage.setPaymentType("1");
                        invoiceManage.setSendStatus("0");
                        invoiceManage.setInvoiceStatus("0");
                        invoiceManage.setInvoiceTax(null);
                        invoiceManage.setRemark(null);
                        invoiceManage.setCreateUser(ShiroUtils.getUserId());
                        invoiceManage.setCreateTime(new Date());
                        invoiceManageService.insertSelective(invoiceManage);
                    }
                }
            }
        }
        contractManageMapper.updateByPrimaryKeySelective(contractManage);
        // 票据管理 新增一条记录
        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(contractManage.getCustomerManagementId());
        ReceiptManage receiptManage = new ReceiptManage();
        receiptManage.setBatchNumber(customerManagement.getBatchNumber());
        receiptManage.setRoomSelectionNumber(customerManagement.getRoomSelectionNumber());
        receiptManage.setBuyerName(customerManagement.getBuyerName());
        receiptManage.setCustomerManagementId(model.getCustomerManagementId());
        receiptManage.setBusinessId(id);
        // 票据类型
        String receiptType = "";
        switch (model.getContentType()){
            case 1: // 定金
                receiptType = "2";
                break;
            case 2: // 首付
                receiptType = "3";
                break;
            case 3: // 银行按揭
                receiptType = "4";
                break;
        }
        receiptManage.setReceiptType(receiptType);
        receiptManage.setReceiptMoney(model.getPayMoney());
        // TODO: 2024/10/21 预收款登记暂无 流水号
//        receiptManage.setReceiptNumber();
        receiptManageService.add(receiptManage);

        return result;
    }


    /**
     * 收款信息
     *
     * @param model
     * @param dt
     * @return
     */
    public List<PayLog> payListAll(PayLog model, DateTrans dt) {
        return modelMapper.selectByExample(getCondition(model, dt));
    }

    private PayLogExample getCondition(PayLog model, DateTrans dt) {
        PayLogExample ex = new PayLogExample();
        ex.setOrderByClause("created_at asc");
        PayLogExample.Criteria criteria = ex.createCriteria();
        if (StringUtils.isNotEmpty(model.getContractId())) {
            criteria.andContractIdEqualTo(model.getContractId());
        }
        if (ObjectUtil.isNotNull(model.getPayType())) {
            criteria.andPayTypeEqualTo(model.getPayType());
        }
        return ex;
    }

    /**
     * 查询合同部分的信息
     *
     * @param contractId
     * @return
     */
    public PayLogDto getContractPart(String contractId) {

        ContractManage contractManage = contractManageMapper.selectByPrimaryKey(contractId);
        PayLogDto payLogDto = new PayLogDto();
        payLogDto.setContractNumber(contractManage.getContractNumber());
        payLogDto.setBuyerName(contractManage.getBuyerName());
        payLogDto.setHouseName(contractManage.getHouseName());
        payLogDto.setPaymentMethod(contractManage.getPaymentMethod());
//        payLogDto.setDeadLine(DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, contractManage.getDeadline()));
        // 查询已收房款
        String receivedMoneyStr = extendMapper.getMoneyByContractId(contractId, 1);
        BigDecimal receivedMoney = new BigDecimal(receivedMoneyStr);
        // 欠款
        BigDecimal arrears = BigDecimal.ZERO;
        // 应收
        BigDecimal receivableMoney = BigDecimal.ZERO;
        if(ObjectUtil.isNotNull(contractManage.getTotalPrice())){
            arrears = contractManage.getTotalPrice().subtract(receivedMoney);
            receivableMoney = contractManage.getTotalPrice();
        }
        payLogDto.setReceivableMoney(receivableMoney);
        payLogDto.setReceivedMoney(receivedMoney);
        payLogDto.setArrears(arrears);
        return payLogDto;
    }

    public PayLog getByContractId(String contractId, Integer payType) {

        PayLogExample payLogExample = new PayLogExample();
        payLogExample.createCriteria().andContractIdEqualTo(contractId).andPayTypeEqualTo(payType);
        List<PayLog> payLogs = modelMapper.selectByExample(payLogExample);
        return payLogs.get(0);

    }

    public PayLogDto getById(String id) {
        PayLog payLog = selectByPrimaryKey(id);
        String username = "";
        if (StringUtils.isNotEmpty(payLog.getCreatedBy())) {
            FrameUser frameUser = frameUserService.selectByPrimaryKey(payLog.getCreatedBy());
            username = frameUser == null ? "" : frameUser.getTruename();
        }
        PayLogDto result = new PayLogDto();
        result.setCreatedName(username);
        BeanUtil.copyProperties(payLog, result);
        return result;
    }

    /**
     * 根据合同id 查询收款
     * @param contractId
     * @param payType
     * @return
     */
    public String getMoneyByContractId(String contractId,Integer payType){
        String money = extendMapper.getMoneyByContractId(contractId, payType);
        return money;
    }


    /**
     * 收款收据
     * @param id
     * @param response
     */
    public void downLoadReceipt(String id, HttpServletResponse response) {

        PayLog payLog = selectByPrimaryKey(id);
        Integer contentType = payLog.getContentType();
        String filePath = "";
        String fileName = "";
        switch (contentType){
            case 1: // 定金
                filePath = "/home/project/settleDown/tem/.docx";
                fileName = "定金收据.docx";
                break;
            case 2: // 首付
                filePath = "/home/project/settleDown/tem/.docx";
                fileName = "首付收据.docx";
                break;
            case 3: // 银行按揭
                filePath = "/home/project/settleDown/tem/.docx";
                fileName = "银行按揭收据.docx";
                break;
        }

        String customerManagementId = payLog.getCustomerManagementId();
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.setOrderByClause("order_num asc");
        buyerExample.createCriteria().andCustomerManagementIdEqualTo(customerManagementId);
        List<Buyer> buyers = buyerService.selectByExample(buyerExample);
        String name = "";
        if (CollectionUtils.isNotEmpty(buyers)) {
            Buyer buyer = buyers.get(0);
            name = buyer.getName();
        }

        ParkRoomResponse roomResponse = roomExtendMapper.getFullName(payLog.getHouseId());
        String fullName = roomResponse.getGroupName()
                + "-"
                + roomResponse.getDiscName()
                + roomResponse.getBuildName()
                + roomResponse.getRoomNo();

        String collectionTime = DateUtils.parseDateToStr(DateUtils.DATE_CH, payLog.getPayTime());
        String content = "";
        String paymentMethod = "";
        String contentTypeLabel = sysDictService.getDictLabel("CONTENT_TYPE",payLog.getContentType().toString());
        String paymentMethodLabel = sysDictService.getDictLabel("COLLECTION_METHODS",payLog.getPaymentMethod().toString());
        if(StringUtils.isNotEmpty(contentTypeLabel)){
            content = contentTypeLabel;
        }
        if(StringUtils.isNotEmpty(paymentMethodLabel)){
            paymentMethod = paymentMethodLabel;
        }
        String receivedAmount = payLog.getPayMoney().toString();
        String capital = MoneyUtils.amountConversion(receivedAmount);

        Map<String, Object> data = new HashMap<>();
        Map<String, Object> picData = new HashMap<>();
        Map<String, Object> sealData = new HashMap<>();
        data.put("${name}", name);
        data.put("${collectionTime}", collectionTime);
        data.put("${fullName}", fullName);
        data.put("${content}", content);
        data.put("${paymentMethod}", paymentMethod);
        data.put("${capital}", capital);
        data.put("${receivedAmount}", receivedAmount);

        ReplaceWord.operateWord(response, filePath, fileName, data, picData, sealData);


    }






}
